# Minimal Decomposition Field of Polynomial over Ratinal
#
#   by Shin-ichiro Hara
#
# Version 1.10 (2001.11.01)

require "polynomial"
require "polynomial-factor"
require "m-polynomial"
require "groebner-basis"
require "residue-class-ring"

module Algebra
  class Polynomial
    def minimal_decomposition_field(fact = nil)
      Algebra.MinimalDecompositionField(self, fact)
    end
  end

  def MinimalDecompositionField(poly, facts1 = nil)
    var_obj = "a"
    ext_field = poly.ground
    polys = [[poly, 1]]
    roots = []
    addelems = []
    modulus = []
    fac = Factors.new

    while fn = polys.shift
      f, n = fn
      if facts1
	facts = facts1
	facts1 = nil
      else
	facts = f.factorize
      end

      first= nil
      rests = []

      facts.each do |f, n|
	if f.deg <= 1
	  fac.push [f, n]
	  if f.deg == 1# && f.ground != poly.ground
	    n.times do
	      roots.push -f[0]
	    end
	  end
	  next
	end
	unless first
	  first = [f, n]
	else
	  rests.push [f, n]
	end
      end
      if first
	f0, n0 = first
	newF = Algebra.AlgebraicExtentionField(ext_field, var_obj) {|v|
	  f0.evaluate(v)
	}
	modulus.push newF.modulus
	new_var = newF.var
	roots.push new_var
	addelems.push new_var

	k = Algebra.Polynomial(newF, "x")
	x = k.var
	h = f0.convert_to(k)
	fac.push [x - new_var, n0]
	q = h / (x - new_var)
	rests.unshift([q, n0])

	ext_field = newF
	var_obj = var_obj.succ
      end
      polys = rests + polys
    end

    [ext_field, modulus, fac, roots, addelems]
  end

  def symmetric_product(objs)
    a = [1]
    objs.each do |x|
      ax = a.collect{|t| t * x}
      a.each_index do |i|
	if i > 0 && i < a.size
	  a[i] = a[i] + ax[i-1]
	end
      end
      a.push ax.last
    end
    a
  end
  
  def MinimalDecompositionField_incomplete(f, facts = f.factorize)
    # this is wrong algorithm
    vars = []
    var = "a"
    facts.each do |f, n|
      dim = f.deg
      next if dim <= 1
      dim.downto 1 do
	vars.unshift var
	var = var.succ
      end
    end
    
    pr = Algebra.MPolynomial(f.ground)
    vs = pr.vars(*vars)
    
    i = 0
    fs = []
    facts.each do |f, n|
      dim = f.deg
      if dim > 1
	sym = symmetric_product(vs[i, dim])
	fa = (1..dim).collect{|j| sym[j] - f[dim-j]*(-1)**j}
	fs.concat fa
	i += dim
      end
    end
    
    pr.sysvar(:basis, Groebner.basis(fs))
    pr.class_eval "
    def inverse
      qs, bs = Groebner.basis_coeff([self] + @@basis)
      unless bs == [1]
	raise self.to_s + \" can't be inverted.\"
      end
      qs[0][0]
    end
    "
    rs = Algebra.ResidueClassRing(pr, pr.basis)
    rs.sysvar(:vars, pr.vars.collect{|v| rs[v]})
    rs
  end

  def MinimalPolynomial(e, *fs)
    raise "too many polys" if fs.size > ("a"..."x").size
    require "groebner-basis"
    (mpring = e.type).vars("x")
    x = mpring.var("x")
    gb = Algebra::Groebner.basis([x - e, * fs])
    gb.last
  end

  module_function :MinimalPolynomial
  module_function :MinimalDecompositionField_incomplete
  module_function :symmetric_product, :MinimalDecompositionField
end

if __FILE__ == $0
  require "rational"
  class Rational < Numeric;def inspect; to_s; end;end

  P = Algebra.Polynomial(Rational, "t")
  t = P.var
#=begin
  f = t**2 - 2
  ext_field, modulus, fac, roots, addelems =
    Algebra.MinimalDecompositionField(f)
  a, b = roots
  p a
  p b
  p 1/(b+2)
  p 1/(b-1)

  puts
  puts

  f = t**3 - 2
  ext_field, modulus, fac, roots, addelems =
    Algebra.MinimalDecompositionField(f)
  a, b, c = roots
  p a + b + c
  p a*b + b*c + c*a
  p a*b*c
  p ((a-2).inverse)
  p ((a-1)*(b-1)*(c-1))
  puts
  puts
#=end
  PQ = Algebra.MPolynomial(Rational)
  a, b, c, d = PQ.vars("abcd")
  p Algebra.MinimalPolynomial(a + b, a**3-2, b**2+b+1)
  p Algebra.MinimalPolynomial(a + b + c, a**2-2, b**2-3, c**2-5)
#  p Algebra.MinimalPolynomial(a + b, a**5-2, b**4+b**3+b**2+b+1)
#x^20 + 5x^19 + 15x^18 + 35x^17 + 70x^16 + 113x^15 + 155x^14 + 215x^13 + 410x^12 + 1095x^11 + 1929x^10 + 2185x^9 + 1180x^8 - 905x^7 - 835x^6 + 17x^5 + 40x^4 - 255x^3 - 45x^2 + 135x + 81

  #too difficult, deg == 16
  #p Algebra.MinimalPolynomial(a+b+c+d, a**2-2, b**2-3, c**2-5, d**2-7)
end
